<?php

declare(strict_types=1);

namespace Internetgalerie\IgCrmTimeRecording\Controller;

use Internetgalerie\IgCrmTimeRecording\Utility\PersonUtility;
use Internetgalerie\IgCrmTimeRecording\Domain\Repository\HolidayRepository;
use Psr\Http\Message\ResponseInterface;
use TYPO3\CMS\Core\Messaging\AbstractMessage;

/**
 * HolidayController
 */
class HolidayController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    protected ?HolidayRepository $holidayRepository = null;
    protected ?PersonUtility $personUtility = null;

    public function injectHolidayRepository(HolidayRepository $holidayRepository)
    {
        $this->holidayRepository = $holidayRepository;
    }
    public function injectPersonUtility(PersonUtility $personUtility)
    {
        $this->personUtility = $personUtility;
    }

    /**
     * action list
     *
     * @return ResponseInterface
     */
    public function listAction(): ResponseInterface
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }
        $holidays = $this->holidayRepository->findAll();
        $this->view->assign('holidays', $holidays);
        return $this->htmlResponse();
    }

    /**
     * action new
     *
     * @return ResponseInterface
     */
    public function newAction(): ResponseInterface
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }        
        return $this->htmlResponse();
    }

    /**
     * action create
     *
     * @param \Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $newHoliday
     */
    public function createAction(\Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $newHoliday)
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }
        $this->addFlashMessage('Eintrag erstellt', '', AbstractMessage::INFO);
        $this->holidayRepository->add($newHoliday);
        return $this->redirect('list');
    }

    /**
     * action edit
     *
     * @param \Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday
     * @TYPO3\CMS\Extbase\Annotation\IgnoreValidation("holiday")
     * @return ResponseInterface
     */
    public function editAction(\Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday): ResponseInterface
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }
        $this->view->assign('holiday', $holiday);
        return $this->htmlResponse();
    }

    /**
     * action update
     *
     * @param \Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday
     */
    public function updateAction(\Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday): ResponseInterface
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }
        $this->addFlashMessage('Eintrag aktualisiert', '', AbstractMessage::INFO);
        $this->holidayRepository->update($holiday);
        return $this->redirect('list');
    }

    /**
     * action delete
     *
     * @param \Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday
     */
    public function deleteAction(\Internetgalerie\IgCrmTimeRecording\Domain\Model\Holiday $holiday): ResponseInterface
    {
        if (!$this->personUtility->hasRoleAdmin()) {
            return $this->htmlResponse('no access');
        }
        $this->addFlashMessage('Eintrag gelöscht', '', AbstractMessage::INFO);
        $this->holidayRepository->remove($holiday);
        return $this->redirect('list');
    }

    public function targetTimesAction(int $year = null): ResponseInterface
    {
        //var_Dump($year);
        $year ??= (int)date('Y');
        $yearNav = [
            'prev' => $year - 1,
            'current' => $year,
            'next' => $year + 1,
        ];
        $this->view->assign('yearNav', $yearNav);
        
        $targetTimes = $this->holidayRepository->getTargetTimesByYear($year, $this->personUtility->getWorkingDayInSeconds());
        $this->view->assign('workingDayInSeconds', $this->personUtility->getWorkingDayInSeconds());
        $this->view->assign('workingDayInHHMM', $this->personUtility->getWorkingDayInHHMM());
        $this->view->assign('targetTimes', $targetTimes);
        $this->view->assign('percents', $this->personUtility->getTargetTimesPercents());
        $this->view->assign('timePerDay', $this->personUtility->getWorkingDayInSeconds());
        $this->view->assign('year', $year);
        //var_dump($targetTimes);exit(0);

        $holidays = $this->holidayRepository->findByYear($year);
        $this->view->assign('holidays', $holidays);
        return $this->htmlResponse();
    }


}
